function [E,U] = AccSMUFS(X, viewU, B, lambda, gamma, featureNumber, neighbor,maxIter)
% Input:
%       - X: the data matrix of size d x n;
%       - viewU: the aligned membership matrices of samples;
%       - B: the graph between samples and anchors of size n x m;
%       - lambda, gamma:  the regularization parameters;
%       - featureNumber: the number of selected features;
%       - neighbor     : the number of neighbor;
% Output:
%       - E: the projection matrix
%       - U: the unified membership matrix

%%
[nFea, nSmp] = size(X);
[~, mAnchor] = size(B);
viewNumber = length(viewU);
C = size(viewU{1},2);


%% Initialization
viewWeight = ones(nSmp, viewNumber)/viewNumber;

U = zeros(size(viewU{1}));
jointU = zeros(size(U));

Id = diag(ones(nFea,1));

for v =1:viewNumber
    U = U + viewU{v}/viewNumber;
    temp = spdiags(viewWeight(:,v), 0, nSmp, nSmp);
    jointU = jointU + temp*viewU{v};
end

ka = diag(sum(B));

XXT = X * X';

E = zeros(nFea, C);
PI = E;
rho = 1.1;
mu = 1;
MatPar = (1+lambda + gamma);

W = inv( XXT + mu/2 * Id ) *  X*U;

for iter = 1:maxIter

    %% updata UA
    UA = ka\B'*jointU;

    %% updata U
    MatU = jointU+gamma*X'*W;
    tempU = MatU/MatPar + B * (inv(ka/lambda - B' * B/MatPar) * (B' * MatU))/MatPar^2;

    for i =1:nSmp
        U(i,:) = EProjSimplex_new(tempU(i,:), 1);
    end

    %% updata viewWeight

    for i =1 :nSmp
        Mat_U_i = zeros(viewNumber,C);
        for v = 1:viewNumber
            Mat_U_i(v,:) = U(i,:)-viewU{v}(i,:);
        end
        retview = SimplexQP(Mat_U_i*Mat_U_i', 0, 200, 1.01, 0, viewWeight(i,:)');
        viewWeight(i,:) = retview';
    end

    jointU = zeros(size(U));
    for v =1:viewNumber
        temp = spdiags(viewWeight(:,v), 0, nSmp, nSmp);
        jointU = jointU + temp*viewU{v};
    end

    %% updata B
    [B,beta] = upData_S(U, UA, lambda, neighbor);
    ka = diag(sum(B));

    %% updata W
    W = inv( gamma * XXT + mu/2 * Id ) * ( gamma * X*U + mu/2*E + PI/2);

    %% updata E
    Q = W - PI/mu;
    Mat_Q = Q*Q';
    [~,index] = sort(diag(Mat_Q),'descend');

    E(index(1:featureNumber),:) = W(index(1:featureNumber),:);
    E(index(featureNumber+1:end),:) = 0;

    if sum(sum(B)==0) ~=0
        break;
    end

    %%  updata ALM
    PI = PI + mu.*(E - W);
    mu = rho * mu;


    T = zeros(nSmp+mAnchor);
    T(1:nSmp,nSmp+1:end) = B;
    T(nSmp+1:end,1:nSmp) = B';
    R = blkdiag(eye(nSmp),ka);
    Ls = R-T;
    UUA = [U;UA];

    %% objective
    obj1(iter) = (norm((U-jointU),'fro'))^2;
    obj2(iter) = lambda * trace(UUA' * Ls * UUA);
    obj3(iter) = beta * trace(B'*B);
    obj4(iter) = gamma * trace((X'*W - U)'*(X'*W - U));
    obj5(iter) = mu/2 * trace((E -W + PI/mu)'*(E -W + PI/mu));

    obj(iter) = obj1(iter)+obj2(iter)+obj3(iter)+obj4(iter)+obj5(iter);
    if iter>2
        change=abs((obj(iter)-obj(iter-1))/obj(iter-1));
        if change<1e-2
            break;
        end
    end
end

end