% construct similarity matrix with probabilistic k-nearest neighbors. It is a parameter free, distance consistent similarity.
function W = constructW_PKN(X, Z, k, issymmetric)
% X: each column is a data point
% k: number of neighbors
% issymmetric: set W = (W+W')/2 if issymmetric=1
% W: similarity matrix

if nargin < 3
    issymmetric = 1;
end
if nargin < 2
    k = 5;
end

[dim, n] = size(X);
m = size(Z,2);
D = L2_distance_1(X, Z);
[dumb, idx] = sort(D, 2); % sort each row

W = zeros(n,m);
for i = 1:n
    id = idx(i,1:k+1);
    di = D(i, id);
    W(i,id) = (di(k+1)-di)/(k*di(k+1)-sum(di(1:k))+eps);
end

if issymmetric == 1
    W = (W+W')/2;
end





